# updater.js Copyright (c) 2013 Cloud Elements.  All rights reserved.
# CLOUD-ELEMENTS PROPRIETARY / CONFIDENTIAL. Use is subject to license
# terms.
# 
# Created on 8/22/14 by Charlie Ozinga.

require! fs
require! async
require! http
require! xml2js
require! child_process: child
require! 'prelude-ls': pl

webapps = '/opt/tomcat/webapps'
mvn = '/opt/maven/bin/mvn'
dbdeploy = '/Users/charlieozinga/Scripts/dbdeploy'
startup = '/opt/tomcat/bin/startup.sh'
shutdown = '/opt/tomcat/bin/shutdown.sh'
basepath = '/cloudelements/elements-application-elements'

doexec = (cmd, ecb, cb) !->
  proc = child.exec cmd, {process.env}, (error, stdout, stderr) ->
    console.log "#cmd stdout: #stdout" if stdout?length
    console.log "#cmd stderr: #stderr" if stderr?length
    console.log "#cmd error: #error" if error?
  proc.on 'exit' (code, sig) !->
    if sig? then ecb "#cmd signal: #sig"
    else if code != 0 then ecb "#cmd code: #code"
    else cb!

with-maven-meta = (path, cb) !->
  process = (data) !->
    xml2js.parse-string data, (err, jsmeta) ->
      cb null, jsmeta
    
  client = http.request do
    * hostname: 'repo.cloud-elements.com'
      port: 80
      path: path
      method: 'GET'
    * (response) ->
        data = ''
        response.set-encoding 'utf8'
        response.on 'readable' -> data += response.read!
        response.on 'error' cb
        response.on 'end' -> process data

  client.on 'error', cb
  client.end!

download-elements-pick = (version, cb) !->
  err, jsmeta <- with-maven-meta basepath + "/#version/maven-metadata.xml"
  cb err,
    jsmeta?metadata.versioning.0.snapshotVersions.0.snapshotVersion.0.value

download-elements = (version, cb) !->
  download-with = (vers) ->
    err <- doexec "rm -rf #webapps/elements", cb
    return cb err if err?
    err <- doexec "rm -f #webapps/elements.war", cb
    return cb err if err?
    local = webapps + '/elements.war'
    dfile = fs.create-write-stream local
    client = http.request do
      * hostname: 'repo.cloud-elements.com'
        port: 80
        path: basepath + "/#version/elements-application-elements-#vers.war"
        method: 'GET'
      * (response) ->
          if response.status-code != 200
            cb "couldn't download #version: #{response.status-code}"
          else
            response.pipe dfile
            dfile.on 'finish' !-> dfile.close cb
    client.on 'error' (err) !-> fs.unlink local ; cb err
    client.end!

  if version.index-of('SNAPSHOT') != -1
    err, vers <- download-elements-pick version 
    return cb err if err?
    console.log "using vers #vers"
    download-with vers
  else 
    download-with version

exports.get-available = (cb) !->
  err, jsmeta <- with-maven-meta basepath + '/maven-metadata.xml'
  cb err, jsmeta?metadata.versioning.0.versions.0.version

exports.get-latest = (cb) !->
  err, jsmeta <- with-maven-meta basepath + '/maven-metadata.xml'
  cb err, jsmeta?metadata.versioning.0.release.0

exports.get-current = (cb) !->
  mf = webapps + "/elements/META-INF/MANIFEST.MF"
  err, data <- fs.read-file mf, encoding: \utf8
  return cb err if err?
  for line in data.split '\n'
    if line.index-of('version: ') == 0
      version = line.substring 9 .trim!
      break
  if version?
    cb null, version
  else
    cb "can't read version from #mf"

exports.set-current = (version, cb) !->
  console.log "shutting down tomcat..."
  <- doexec shutdown, cb
  <- set-timeout _, 4000
  console.log "updating WAR resource..."
  err <- download-elements version
  return cb err if err?
  console.log "updating database..."
  <- doexec dbdeploy, cb
  console.log "restarting tomcat..."
  <- doexec startup, cb

